@@ -411,12 +411,18 @@ set_message_annotations(
     Anns1 = #'v1_0.message_annotations'{content = maps:to_list(Anns)},
     Msg#amqp10_msg{message_annotations = Anns1}.
 
+wrap_ap_value(true) ->
+    {boolean, true};
+wrap_ap_value(false) ->
+    {boolean, false};
 wrap_ap_value(V) when is_integer(V) ->
     {uint, V};
 wrap_ap_value(V) when is_binary(V) ->
     utf8(V);
 wrap_ap_value(V) when is_list(V) ->
-    utf8(list_to_binary(V)).
+    utf8(list_to_binary(V));
+wrap_ap_value(V) when is_atom(V) ->
+    utf8(atom_to_list(V)).
 
 
 %% LOCAL
