@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
 %%
 
 %% @private
@@ -54,12 +54,14 @@ start_infrastructure_fun(Sup, Conn, network) ->
     fun (Sock, ConnName) ->
             {ok, ChMgr} = start_channels_manager(Sup, Conn, ConnName, network),
             {ok, AState} = rabbit_command_assembler:init(?PROTOCOL),
+            {ok, GCThreshold} = application:get_env(amqp_client, writer_gc_threshold),
             {ok, Writer} =
                 supervisor2:start_child(
                   Sup,
                   {writer,
                    {rabbit_writer, start_link,
-                    [Sock, 0, ?FRAME_MIN_SIZE, ?PROTOCOL, Conn, ConnName]},
+                    [Sock, 0, ?FRAME_MIN_SIZE, ?PROTOCOL, Conn, ConnName,
+                     false, GCThreshold]},
                    transient, ?WORKER_WAIT, worker, [rabbit_writer]}),
             {ok, Reader} =
                 supervisor2:start_child(
