@@ -121,7 +121,7 @@ start_in(DataDir) ->
 
 %% @doc Restarts a previously successfully started ra server
 %% @param ServerId the ra_server_id() of the server
-%% @returns `{ok | error, Error}' when error can be
+%% @returns `{ok | error, Error}' where error can be
 %% `not_found' or `name_not_registered' when the ra server has never before
 %% been started on the Erlang node.
 %% @end
@@ -155,9 +155,9 @@ stop_server(ServerId) ->
 %% @doc Deletes a ra server
 %% The server is forcefully deleted.
 %% @param ServerId the ra_server_id() of the server
-%% @returns `{ok | error, nodedown}'
+%% @returns `ok | {error, nodedown} | {badrpc, Reason}'
 %% @end
--spec force_delete_server(ServerId :: ra_server_id()) -> ok | {error, term()}.
+-spec force_delete_server(ServerId :: ra_server_id()) -> ok | {error, term()} | {badrpc, term()}.
 force_delete_server(ServerId) ->
     ra_server_sup_sup:delete_server(ServerId).
 
@@ -535,7 +535,7 @@ leave_and_terminate(ServerId) ->
 %% @param ServerId the ra server to remove
 %% @see leave_and_terminate/3
 %% @end
--spec leave_and_terminate(ra_server_id(), ra_server_id()) ->
+-spec leave_and_terminate(ra_server_id() | [ra_server_id()], ra_server_id()) ->
     ok | timeout | {error, noproc}.
 leave_and_terminate(ServerRef, ServerId) ->
     leave_and_terminate(ServerRef, ServerId, ?DEFAULT_TIMEOUT).
@@ -546,7 +546,8 @@ leave_and_terminate(ServerRef, ServerId) ->
 %% @param Timeout timeout to use
 %% @see leave_and_terminate/2
 %% @end
--spec leave_and_terminate(ra_server_id(), ra_server_id(), timeout()) ->
+-spec leave_and_terminate(ra_server_id() | [ra_server_id()],
+                          ra_server_id(), timeout()) ->
     ok | timeout | {error, noproc}.
 leave_and_terminate(ServerRef, ServerId, Timeout) ->
     LeaveCmd = {'$ra_leave', ServerId, await_consensus},
@@ -650,7 +651,7 @@ overview() ->
 %% @end
 -spec process_command(ServerId :: ra_server_id() | [ra_server_id()],
                       Command :: term(),
-                      Timeout :: non_neg_integer()) ->
+                      Timeout :: timeout()) ->
     {ok, Reply :: term(), Leader :: ra_server_id()} |
     {error, term()} |
     {timeout, ra_server_id()}.
@@ -689,7 +690,7 @@ process_command(ServerId, Command) ->
 %% structure will be returned informing the caller that it cannot process the
 %% message. The message will include the current cluster leader, if one is known:
 %%
-%% `{ra_event, CurrentLeader, {rejected, {not_leader, Leader, Correlation}}}'
+%% `{ra_event, FromId, {rejected, {not_leader, Leader | undefined, Correlation}}}'
 %%
 %% The caller must then redirect the command for the correlation identifier to
 %% the correct ra server: the leader.
@@ -780,7 +781,8 @@ local_query(ServerId, QueryFun) ->
 -spec local_query(ServerId :: ra_server_id(),
                   QueryFun :: query_fun(),
                   Timeout :: timeout()) ->
-    ra_server_proc:ra_leader_call_ret({ra_idxterm(), term()}).
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), term()}) |
+    {ok, {ra_idxterm(), term()}, not_known}.
 local_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, local, Timeout).
 
@@ -791,25 +793,27 @@ local_query(ServerId, QueryFun, Timeout) ->
 %% The leader state may be more up-to-date compared to local state of some followers.
 %% This function may still return stale results as it reads the current state
 %% and does not wait for commands to be applied.
-%% @param ServerId the ra server id to send the query to
+%% @param ServerId the ra server id(s) to send the query to
 %% @param QueryFun the query function to run
 %% @end
--spec leader_query(ServerId :: ra_server_id(),
+-spec leader_query(ServerId :: ra_server_id() | [ra_server_id()],
                    QueryFun :: query_fun()) ->
-    {ok, {ra_idxterm(), term()}, ra_server_id() | not_known}.
+    ra_server_proc:ra_leader_call_ret(term()) |
+    {ok, {ra_idxterm(), term()}, not_known}.
 leader_query(ServerId, QueryFun) ->
     leader_query(ServerId, QueryFun, ?DEFAULT_TIMEOUT).
 
 %% @doc Same as `leader_query/2' but accepts a custom timeout.
-%% @param ServerId the ra server id to send the query to
+%% @param ServerId the ra server id(s) to send the query to
 %% @param QueryFun the query function to run
 %% @param Timeout the timeout to use
 %% @see leader_query/2
 %% @end
--spec leader_query(ServerId :: ra_server_id(),
+-spec leader_query(ServerId :: ra_server_id() | [ra_server_id()],
                    QueryFun :: query_fun(),
                    Timeout :: timeout()) ->
-    {ok, {ra_idxterm(), term()}, ra_server_id() | not_known}.
+    ra_server_proc:ra_leader_call_ret(term()) |
+    {ok, {ra_idxterm(), term()}, not_known}.
 leader_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, leader, Timeout).
 
@@ -819,22 +823,22 @@ leader_query(ServerId, QueryFun, Timeout) ->
 %% Consistency guarantee is that the query will return result containing
 %% at least all changes, committed before this query is issued.
 %% This may include changes which were committed while the query is running.
-%% @param ServerId the ra server id to send the query to
+%% @param ServerId the ra server id(s) to send the query to
 %% @param QueryFun the query function to run
 %% @end
--spec consistent_query(ServerId :: ra_server_id(),
+-spec consistent_query(ServerId :: ra_server_id() | [ra_server_id()],
                        QueryFun :: query_fun()) ->
     ra_server_proc:ra_leader_call_ret(term()).
 consistent_query(ServerId, QueryFun) ->
     consistent_query(ServerId, QueryFun, ?DEFAULT_TIMEOUT).
 
 %% @doc Same as `consistent_query/2' but accepts a custom timeout.
-%% @param ServerId the ra server id to send the query to
+%% @param ServerId the ra server id(s) to send the query to
 %% @param QueryFun the query function to run
 %% @param Timeout the timeout to use
 %% @see consistent_query/2
 %% @end
--spec consistent_query(ServerId :: ra_server_id(),
+-spec consistent_query(ServerId :: ra_server_id() | [ra_server_id()],
                        QueryFun :: query_fun(),
                        Timeout :: timeout()) ->
     ra_server_proc:ra_leader_call_ret(term()).
@@ -842,28 +846,28 @@ consistent_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, consistent, Timeout).
 
 %% @doc Returns a list of cluster members
-%% @param ServerId the Ra server to send the query to
+%% @param ServerId the Ra server(s) to send the query to
 %% @end
--spec members(ra_server_id()) ->
+-spec members(ra_server_id() | [ra_server_id()]) ->
     ra_server_proc:ra_leader_call_ret([ra_server_id()]).
 members(ServerId) ->
     members(ServerId, ?DEFAULT_TIMEOUT).
 
 %% @doc Returns a list of cluster members
-%% @param ServerId the Ra server to send the query to
+%% @param ServerId the Ra server(s) to send the query to
 %% @param Timeout the timeout to use
 %% @end
--spec members(ra_server_id(), timeout()) ->
+-spec members(ra_server_id() | [ra_server_id()], timeout()) ->
     ra_server_proc:ra_leader_call_ret([ra_server_id()]).
 members(ServerId, Timeout) ->
     ra_server_proc:state_query(ServerId, members, Timeout).
 
--spec initial_members(ra_server_id()) ->
-    ra_server_proc:ra_leader_call_ret([ra_server_id()]).
+-spec initial_members(ra_server_id() | [ra_server_id()]) ->
+    ra_server_proc:ra_leader_call_ret([ra_server_id()] | error).
 initial_members(ServerId) ->
     initial_members(ServerId, ?DEFAULT_TIMEOUT).
 
--spec initial_members(ra_server_id(), timeout()) ->
+-spec initial_members(ra_server_id() | [ra_server_id()], timeout()) ->
     ra_server_proc:ra_leader_call_ret([ra_server_id()] | error).
 initial_members(ServerId, Timeout) ->
     ra_server_proc:state_query(ServerId, initial_members, Timeout).
@@ -872,7 +876,7 @@ initial_members(ServerId, Timeout) ->
 %% Returns `already_leader' if the transfer targer is already the leader.
 %% @end
 -spec transfer_leadership(ra_server_id(), ra_server_id()) ->
-    ok | already_leader.
+    ok | already_leader | {error, term()} | {timeout, ra_server_id()}.
 transfer_leadership(ServerId, TargetServerId) ->
     ra_server_proc:transfer_leadership(ServerId, TargetServerId, ?DEFAULT_TIMEOUT).
 
