@@ -54,7 +54,6 @@
 -define(DEFAULT_STOP_FOLLOWER_ELECTION, false).
 -define(DEFAULT_AWAIT_CONDITION_TIMEOUT, 30000).
 %% Utilisation average calculations are all in μs.
--define(USE_AVG_HALF_LIFE, 1000000.0).
 -define(INSTALL_SNAP_RPC_TIMEOUT, 120 * 1000).
 -define(DEFAULT_RECEIVE_SNAPSHOT_TIMEOUT, 30000).
 -define(DEFAULT_SNAPSHOT_CHUNK_SIZE, 1000000). % 1MB
@@ -162,7 +161,7 @@ cast_command(ServerId, Priority, Cmd) ->
 -spec query(server_loc(), query_fun(),
             local | consistent | leader, timeout()) ->
     ra_server_proc:ra_leader_call_ret(term())
-    | {ok, Reply :: term(), ra_server_id() | not_known}.
+    | {ok, Reply :: term(), not_known}.
 query(ServerLoc, QueryFun, local, Timeout) ->
     statem_call(ServerLoc, {local_query, QueryFun}, Timeout);
 query(ServerLoc, QueryFun, leader, Timeout) ->
@@ -238,8 +237,10 @@ multi_statem_call([ServerId | ServerIds], Msg, Errs, Timeout) ->
 init(Config0 = #{id := Id, cluster_name := ClusterName}) ->
     process_flag(trap_exit, true),
     Config = maps:merge(config_defaults(), Config0),
-    #{id := {_, UId, LogId},
-      cluster := Cluster} = ServerState = ra_server:init(Config),
+    #{cluster := Cluster} = ServerState = ra_server:init(Config),
+    LogId = ra_server:log_id(ServerState),
+    UId = ra_server:uid(ServerState),
+    Id = ra_server:id(ServerState),
     Key = ra_lib:ra_server_id_to_local_name(Id),
     % ensure ra_directory has the new pid
     yes = ra_directory:register_name(UId, self(),
@@ -259,8 +260,8 @@ init(Config0 = #{id := Id, cluster_name := ClusterName}) ->
     TickTime = maps:get(tick_timeout, Config),
     AwaitCondTimeout = maps:get(await_condition_timeout, Config),
     RaEventFormatterMFA = maps:get(ra_event_formatter, Config, undefined),
-    FlushCommandsSize = maps:get(low_priority_commands_flush_size, Config,
-                                  ?FLUSH_COMMANDS_SIZE),
+    FlushCommandsSize = application:get_env(ra, low_priority_commands_flush_size,
+                                            ?FLUSH_COMMANDS_SIZE),
     SnapshotChunkSize = application:get_env(ra, snapshot_chunk_size,
                                     ?DEFAULT_SNAPSHOT_CHUNK_SIZE),
     ReceiveSnapshotTimeout = application:get_env(ra, receive_snapshot_timeout,
@@ -731,6 +732,11 @@ terminating_follower(EvtType, Msg, State0) ->
     {State, Actions} = case follower(EvtType, Msg, State0) of
                            {next_state, terminating_follower, S, A} ->
                                {S, A};
+                           {next_state, NextState, S, A} ->
+                               ?DEBUG("~s: terminating follower requested state '~s'"
+                                      " - remaining in current state",
+                                      [LogName, NextState]),
+                               {S, A};
                            {keep_state, S, A} ->
                                {S, A}
                        end,
@@ -941,13 +947,10 @@ handle_receive_snapshot(Msg, State) ->
 handle_await_condition(Msg, State) ->
     handle_raft_state(?FUNCTION_NAME, Msg, State).
 
-perform_local_query(QueryFun, Leader, #{effective_machine_module := MacMod,
-                                        machine_state := MacState,
-                                        last_applied := Last,
-                                        current_term := Term}) ->
-    try ra_machine:query(MacMod, QueryFun, MacState) of
+perform_local_query(QueryFun, Leader, ServerState) ->
+    try ra_server:machine_query(QueryFun, ServerState) of
         Result ->
-            {ok, {{Last, Term}, Result}, Leader}
+            {ok, Result, Leader}
     catch
         _:_ = Err ->
             {error, Err}
@@ -1166,12 +1169,12 @@ send_rpc(To, Msg) ->
 gen_cast(To, Msg) ->
     send(To, {'$gen_cast', Msg}).
 
-send_ra_event(To, Msg, LeaderId, EvtType, State) ->
-    send(To, wrap_ra_event(State, LeaderId, EvtType, Msg)).
+send_ra_event(To, Msg, FromId, EvtType, State) ->
+    send(To, wrap_ra_event(State, FromId, EvtType, Msg)).
 
 wrap_ra_event(#state{conf = #conf{ra_event_formatter = undefined}},
-              LeaderId, EvtType, Evt) ->
-    {ra_event, LeaderId, {EvtType, Evt}};
+              FromId, EvtType, Evt) ->
+    {ra_event, FromId, {EvtType, Evt}};
 wrap_ra_event(#state{conf = #conf{ra_event_formatter = {M, F, A}}},
               LeaderId, EvtType, Evt) ->
     apply(M, F, [LeaderId, {EvtType, Evt} | A]).
@@ -1326,7 +1329,7 @@ reject_command(Pid, Corr, State) ->
                   "Rejecting to ~w ~n", [log_id(State), Pid, LeaderId])
     end,
     send_ra_event(Pid, {not_leader, LeaderId, Corr},
-                  LeaderId, rejected, State).
+                  id(State), rejected, State).
 
 maybe_persist_last_applied(#state{server_state = NS} = State) ->
      State#state{server_state = ra_server:persist_last_applied(NS)}.
