@@ -63,8 +63,9 @@ quorum_queue_migration(_FeatureName, _FeatureProps, is_enabled) ->
     mnesia:table_info(rabbit_durable_queue, attributes) =:= Fields.
 
 migrate_to_amqqueue_with_type(FeatureName, [Table | Rest], Fields) ->
-    rabbit_log:info("Feature flag `~s`:   migrating Mnesia table ~s...",
-                    [FeatureName, Table]),
+    rabbit_log_feature_flags:info(
+      "Feature flag `~s`:   migrating Mnesia table ~s...",
+      [FeatureName, Table]),
     Fun = fun(Queue) -> amqqueue:upgrade_to(amqqueue_v2, Queue) end,
     case mnesia:transform_table(Table, Fun, Fields) of
         {atomic, ok}      -> migrate_to_amqqueue_with_type(FeatureName,
@@ -73,8 +74,9 @@ migrate_to_amqqueue_with_type(FeatureName, [Table | Rest], Fields) ->
         {aborted, Reason} -> {error, Reason}
     end;
 migrate_to_amqqueue_with_type(FeatureName, [], _) ->
-    rabbit_log:info("Feature flag `~s`:   Mnesia tables migration done",
-                    [FeatureName]),
+    rabbit_log_feature_flags:info(
+      "Feature flag `~s`:   Mnesia tables migration done",
+      [FeatureName]),
     ok.
 
 %% -------------------------------------------------------------------
@@ -96,10 +98,10 @@ implicit_default_bindings_migration(_Feature_Name, _FeatureProps,
 remove_explicit_default_bindings(_FeatureName, []) ->
     ok;
 remove_explicit_default_bindings(FeatureName, Queues) ->
-    rabbit_log:info("Feature flag `~s`:   deleting explicit "
-                    "default bindings for ~b queues "
-                    "(it may take some time)...",
-                    [FeatureName, length(Queues)]),
+    rabbit_log_feature_flags:info(
+      "Feature flag `~s`:   deleting explicit default bindings "
+      "for ~b queues (it may take some time)...",
+      [FeatureName, length(Queues)]),
     [rabbit_binding:remove_default_exchange_binding_rows_of(Q)
      || Q <- Queues],
     ok.
