@@ -36,6 +36,8 @@ node(Node, Timeout) ->
 -spec local() -> ok | {error_string, string()}.
 
 local() ->
+    rabbit_log:warning("rabbitmqctl node_health_check and its HTTP API counterpart are DEPRECATED. "
+                       "See https://www.rabbitmq.com/monitoring.html#health-checks for replacement options."),
     run_checks([list_channels, list_queues, alarms, rabbit_node_monitor]).
 
 %%----------------------------------------------------------------------------
