@@ -1,3 +1,19 @@
+%% The contents of this file are subject to the Mozilla Public License
+%% Version 1.1 (the "License"); you may not use this file except in
+%% compliance with the License. You may obtain a copy of the License
+%% at https://www.mozilla.org/MPL/
+%%
+%% Software distributed under the License is distributed on an "AS IS"
+%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+%% the License for the specific language governing rights and
+%% limitations under the License.
+%%
+%% The Original Code is RabbitMQ.
+%%
+%% The Initial Developer of the Original Code is GoPivotal, Inc.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
+%%
+
 -module(rabbit_hipe).
 
 %% HiPE compilation uses multiple cores anyway, but some bits are
@@ -20,17 +36,18 @@ maybe_hipe_compile() ->
     end.
 
 log_hipe_result({ok, disabled}) ->
-    ok;
+    rabbit_log_prelaunch:info(
+      "HiPE disabled: no modules were natively recompiled.~n", []);
 log_hipe_result({ok, already_compiled}) ->
-    rabbit_log:info(
+    rabbit_log_prelaunch:info(
       "HiPE in use: modules already natively compiled.~n", []);
 log_hipe_result({ok, Count, Duration}) ->
-    rabbit_log:info(
+    rabbit_log_prelaunch:info(
       "HiPE in use: compiled ~B modules in ~Bs.~n", [Count, Duration]);
 log_hipe_result(false) ->
     io:format(
       "~nNot HiPE compiling: HiPE not found in this Erlang installation.~n"),
-    rabbit_log:warning(
+    rabbit_log_prelaunch:warning(
       "Not HiPE compiling: HiPE not found in this Erlang installation.~n").
 
 hipe_compile() ->
