@@ -198,11 +198,7 @@ cancel_consumer_handler(QName, {ConsumerTag, ChPid}) ->
 
 cancel_consumer(QName, ChPid, ConsumerTag) ->
     catch rabbit_core_metrics:consumer_deleted(ChPid, ConsumerTag, QName),
-    rabbit_event:notify(consumer_deleted,
-                        [{consumer_tag, ConsumerTag},
-                         {channel,      ChPid},
-                         {queue,        QName},
-                         {user_who_performed_action, ?INTERNAL_USER}]).
+    emit_consumer_deleted(ChPid, ConsumerTag, QName, ?INTERNAL_USER).
 
 local_or_remote_handler(ChPid, Module, Function, Args) ->
     Node = node(ChPid),
@@ -618,7 +614,8 @@ basic_get(Q, NoAck, CTag0, QState0) when ?amqqueue_is_quorum(Q) ->
                     rabbit_types:ctag(), ExclusiveConsume :: boolean(),
                     Args :: rabbit_framing:amqp_table(), ActingUser :: binary(),
                     any(), rabbit_fifo_client:state()) ->
-    {'ok', rabbit_fifo_client:state()}.
+    {'ok', rabbit_fifo_client:state()} |
+    {error, timeout | term()}.
 
 basic_consume(Q, NoAck, ChPid,
               ConsumerPrefetchCount, ConsumerTag0, ExclusiveConsume, Args,
@@ -636,7 +633,8 @@ basic_consume(Q, NoAck, ChPid,
                    Other -> Other
                end,
     %% consumer info is used to describe the consumer properties
-    ConsumerMeta = #{ack => not NoAck,
+    AckRequired = not NoAck,
+    ConsumerMeta = #{ack => AckRequired,
                      prefetch => ConsumerPrefetchCount,
                      args => Args,
                      username => ActingUser},
@@ -647,7 +645,6 @@ basic_consume(Q, NoAck, ChPid,
     case ra:local_query(QPid,
                         fun rabbit_fifo:query_single_active_consumer/1) of
         {ok, {_, SacResult}, _} ->
-
             SingleActiveConsumerOn = single_active_consumer_on(Q),
             {IsSingleActiveConsumer, ActivityStatus} = case {SingleActiveConsumerOn, SacResult} of
                                                            {false, _} ->
@@ -657,12 +654,14 @@ basic_consume(Q, NoAck, ChPid,
                                                            _ ->
                                                                {false, waiting}
                                                        end,
-
-            %% TODO: emit as rabbit_fifo effect
-            rabbit_core_metrics:consumer_created(ChPid, ConsumerTag, ExclusiveConsume,
-                                                 not NoAck, QName,
-                                                 ConsumerPrefetchCount, IsSingleActiveConsumer,
-                                                 ActivityStatus, Args),
+            rabbit_core_metrics:consumer_created(
+                    ChPid, ConsumerTag, ExclusiveConsume,
+                    AckRequired, QName,
+                    ConsumerPrefetchCount, IsSingleActiveConsumer,
+                    ActivityStatus, Args),
+            emit_consumer_created(ChPid, ConsumerTag, ExclusiveConsume,
+                    AckRequired, QName, Prefetch,
+                    Args, none, ActingUser),
             {ok, QState};
         {error, Error} ->
             Error;
@@ -677,6 +676,25 @@ basic_cancel(ConsumerTag, ChPid, OkMsg, QState0) ->
     maybe_send_reply(ChPid, OkMsg),
     rabbit_fifo_client:cancel_checkout(quorum_ctag(ConsumerTag), QState0).
 
+emit_consumer_created(ChPid, CTag, Exclusive, AckRequired, QName, PrefetchCount, Args, Ref, ActingUser) ->
+    rabbit_event:notify(consumer_created,
+                        [{consumer_tag,   CTag},
+                         {exclusive,      Exclusive},
+                         {ack_required,   AckRequired},
+                         {channel,        ChPid},
+                         {queue,          QName},
+                         {prefetch_count, PrefetchCount},
+                         {arguments,      Args},
+                         {user_who_performed_action, ActingUser}],
+                        Ref).
+
+emit_consumer_deleted(ChPid, ConsumerTag, QName, ActingUser) ->
+    rabbit_event:notify(consumer_deleted,
+        [{consumer_tag, ConsumerTag},
+            {channel, ChPid},
+            {queue, QName},
+            {user_who_performed_action, ActingUser}]).
+
 -spec stateless_deliver(amqqueue:ra_server_id(), rabbit_types:delivery()) -> 'ok'.
 
 stateless_deliver(ServerId, Delivery) ->
