@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_misc).
@@ -58,7 +58,10 @@
 -export([version_minor_equivalent/2, strict_version_minor_equivalent/2]).
 -export([dict_cons/3, orddict_cons/3, maps_cons/3, gb_trees_cons/3]).
 -export([gb_trees_fold/3, gb_trees_foreach/2]).
--export([all_module_attributes/1, build_acyclic_graph/3]).
+-export([all_module_attributes/1,
+         rabbitmq_related_module_attributes/1,
+         module_attributes_from_apps/2,
+         build_acyclic_graph/3]).
 -export([const/1]).
 -export([ntoa/1, ntoab/1]).
 -export([is_process_alive/1]).
@@ -939,11 +942,30 @@ module_attributes(Module) ->
     end.
 
 all_module_attributes(Name) ->
+    Apps = [App || {App, _, _} <- application:loaded_applications()],
+    module_attributes_from_apps(Name, Apps).
+
+rabbitmq_related_module_attributes(Name) ->
+    Apps = rabbitmq_related_apps(),
+    module_attributes_from_apps(Name, Apps).
+
+rabbitmq_related_apps() ->
+    [App
+     || {App, _, _} <- application:loaded_applications(),
+        %% Only select RabbitMQ-related applications.
+        App =:= rabbit_common orelse
+        App =:= rabbitmq_prelaunch orelse
+        App =:= rabbit orelse
+        lists:member(
+          rabbit,
+          element(2, application:get_key(App, applications)))].
+
+module_attributes_from_apps(Name, Apps) ->
     Targets =
         lists:usort(
           lists:append(
             [[{App, Module} || Module <- Modules] ||
-                {App, _, _}   <- application:loaded_applications(),
+                App           <- Apps,
                 {ok, Modules} <- [application:get_key(App, modules)]])),
     lists:foldl(
       fun ({App, Module}, Acc) ->
@@ -1139,12 +1161,29 @@ is_os_process_alive(Pid) ->
                             run_ps(Pid) =:= 0
                     end},
              {win32, fun () ->
-                             Cmd = "tasklist /nh /fi \"pid eq " ++
-                                 rabbit_data_coercion:to_list(Pid) ++ "\" ",
-                             Res = os_cmd(Cmd ++ "2>&1"),
-                             case re:run(Res, "erl\\.exe", [{capture, none}]) of
-                                 match -> true;
-                                 _     -> false
+                             PidS = rabbit_data_coercion:to_list(Pid),
+                             case os:find_executable("tasklist.exe") of
+                                 false ->
+                                     Cmd =
+                                     format(
+                                       "PowerShell -Command "
+                                       "\"(Get-Process -Id ~s).ProcessName\"",
+                                       [PidS]),
+                                     Res =
+                                     os_cmd(Cmd ++ " 2>&1") -- [$\r, $\n],
+                                     case Res of
+                                         "erl"  -> true;
+                                         "werl" -> true;
+                                         _      -> false
+                                     end;
+                                 _ ->
+                                     Cmd =
+                                     "tasklist /nh /fi "
+                                     "\"pid eq " ++ PidS ++ "\"",
+                                     Res = os_cmd(Cmd ++ " 2>&1"),
+                                     match =:= re:run(Res,
+                                                      "erl\\.exe",
+                                                      [{capture, none}])
                              end
                      end}]).
 
