@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_nodes_common).
@@ -26,7 +26,7 @@
 %% API
 %%
 
--export([make/1, parts/1, names/1, ensure_epmd/0, is_running/2, is_process_running/2]).
+-export([make/1, parts/1, names/1, name_type/1, ensure_epmd/0, is_running/2, is_process_running/2]).
 -export([cookie_hash/0, epmd_port/0, diagnostics/1]).
 
 -spec make({string(), string()} | string()) -> node().
@@ -52,7 +52,10 @@ names(Hostname) ->
         {'DOWN', MRef, process, Pid, Reason} -> {error, Reason}
     end.
 
-make({Prefix, Suffix}) -> list_to_atom(lists:append([Prefix, "@", Suffix]));
+make({Prefix, Suffix}) -> rabbit_data_coercion:to_atom(
+                            lists:append([rabbit_data_coercion:to_list(Prefix),
+                                          "@",
+                                          rabbit_data_coercion:to_list(Suffix)]));
 make(NodeStr)          -> make(parts(NodeStr)).
 
 parts(Node) when is_atom(Node) ->
@@ -64,6 +67,13 @@ parts(NodeStr) ->
         {Prefix, Suffix} -> {Prefix, tl(Suffix)}
     end.
 
+name_type(Node) ->
+    {_, HostPart} = parts(Node),
+    case lists:member($., HostPart) of
+        false -> shortnames;
+        true  -> longnames
+    end.
+
 epmd_port() ->
     case init:get_argument(epmd_port) of
         {ok, [[Port | _] | _]} when is_list(Port) -> Port;
