@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_pbe).
@@ -20,6 +20,8 @@
 -export([encrypt_term/5, decrypt_term/5]).
 -export([encrypt/5, decrypt/5]).
 
+-export_type([encryption_result/0]).
+
 supported_ciphers() ->
     credentials_obfuscation_pbe:supported_ciphers().
 
@@ -41,15 +43,21 @@ default_iterations() ->
 encrypt_term(Cipher, Hash, Iterations, PassPhrase, Term) ->
     credentials_obfuscation_pbe:encrypt_term(Cipher, Hash, Iterations, PassPhrase, Term).
 
-decrypt_term(Cipher, Hash, Iterations, PassPhrase, Base64Binary) ->
-    credentials_obfuscation_pbe:decrypt_term(Cipher, Hash, Iterations, PassPhrase, Base64Binary).
+decrypt_term(_Cipher, _Hash, _Iterations, _PassPhrase, {plaintext, Term}) ->
+    Term;
+decrypt_term(Cipher, Hash, Iterations, PassPhrase, {encrypted, _Base64Binary}=Encrypted) ->
+    credentials_obfuscation_pbe:decrypt_term(Cipher, Hash, Iterations, PassPhrase, Encrypted).
+
+-type encryption_result() :: {'encrypted', binary()} | {'plaintext', binary()}.
 
 -spec encrypt(crypto:block_cipher(), crypto:hash_algorithms(),
-    pos_integer(), iodata(), binary()) -> binary().
+    pos_integer(), iodata() | '$pending-secret', binary()) -> encryption_result().
 encrypt(Cipher, Hash, Iterations, PassPhrase, ClearText) ->
     credentials_obfuscation_pbe:encrypt(Cipher, Hash, Iterations, PassPhrase, ClearText).
 
 -spec decrypt(crypto:block_cipher(), crypto:hash_algorithms(),
-    pos_integer(), iodata(), binary()) -> binary().
-decrypt(Cipher, Hash, Iterations, PassPhrase, Base64Binary) ->
-    credentials_obfuscation_pbe:decrypt(Cipher, Hash, Iterations, PassPhrase, Base64Binary).
\ No newline at end of file
+    pos_integer(), iodata(), encryption_result()) -> any().
+decrypt(_Cipher, _Hash, _Iterations, _PassPhrase, {plaintext, Term}) ->
+    Term;
+decrypt(Cipher, Hash, Iterations, PassPhrase, {encrypted, _Base64Binary}=Encrypted) ->
+    credentials_obfuscation_pbe:decrypt(Cipher, Hash, Iterations, PassPhrase, Encrypted).
