@@ -322,8 +322,12 @@ handle_1_0_frame(Mode, Channel, Payload, State) ->
     catch
         _:#'v1_0.error'{} = Reason ->
             handle_exception(State, 0, Reason);
-        _:Reason ->
-            Trace = erlang:get_stacktrace(),
+        _:{error, {not_allowed, Username}} ->
+            %% section 2.8.15 in http://docs.oasis-open.org/amqp/core/v1.0/os/amqp-core-complete-v1.0-os.pdf
+            handle_exception(State, 0, error_frame(
+                                         ?V_1_0_AMQP_ERROR_UNAUTHORIZED_ACCESS,
+                                         "Access for user '~s' was refused: insufficient permissions~n", [Username]));
+        _:Reason:Trace ->
             handle_exception(State, 0, error_frame(
                                          ?V_1_0_AMQP_ERROR_INTERNAL_ERROR,
                                          "Reader error: ~p~n~p~n",
@@ -418,6 +422,12 @@ handle_1_0_connection_frame(#'v1_0.open'{ max_frame_size = ClientFrameMax,
                          heartbeater = Heartbeater,
                          queue_collector = Collector}
         end,
+    HostnameVal = case Hostname of
+                    undefined -> undefined;
+                    {utf8, Val} -> Val
+                  end,
+    rabbit_log:debug("AMQP 1.0 connection.open frame: hostname = ~s, extracted vhost = ~s, idle_timeout = ~p" ,
+                    [HostnameVal, vhost(Hostname), HeartbeatSec * 1000]),
     %% TODO enforce channel_max
     ok = send_on_channel0(
            Sock,
@@ -703,6 +713,8 @@ send_to_new_1_0_session(Channel, Frame, State) ->
             put({ch_fr_pid, ChFrPid}, {channel, Channel}),
             ok = rabbit_amqp1_0_session:process_frame(ChFrPid, Frame);
         {error, {not_allowed, _}} ->
+            rabbit_log:error("AMQP 1.0: user '~s' is not allowed to access virtual host '~s'",
+                [User#user.username, vhost(Hostname)]),
             %% Let's skip the supervisor trace, this is an expected error
             throw({error, {not_allowed, User#user.username}});
         {error, _} = E ->
@@ -785,14 +797,10 @@ ssl_info(F, #v1{sock = Sock}) ->
         {error, _}  -> '';
         {ok, Items} ->
             P = proplists:get_value(protocol, Items),
-            CS = proplists:get_value(cipher_suite, Items),
-            %% The first form is R14.
-            %% The second is R13 - the extra term is exportability (by
-            %% inspection, the docs are wrong).
-            case CS of
-                {K, C, H}    -> F({P, {K, C, H}});
-                {K, C, H, _} -> F({P, {K, C, H}})
-            end
+            #{cipher := C,
+              key_exchange := K,
+              mac := H} = proplists:get_value(selected_cipher_suite, Items),
+            F({P, {K, C, H}})
     end.
 
 cert_info(F, #v1{sock = Sock}) ->
