@@ -20,6 +20,7 @@ defmodule RabbitMQ.CLI.Core.Config do
     FormatterBehaviour,
     PrinterBehaviour
   }
+  alias RabbitMQ.CLI.Core.Helpers
 
   #
   # Environment
@@ -28,7 +29,7 @@ defmodule RabbitMQ.CLI.Core.Config do
   def get_option(name, opts \\ %{}) do
     raw_option =
       opts[name] ||
-        get_system_option(name) ||
+        get_system_option(name, opts) ||
         default(name)
 
     normalise(name, raw_option)
@@ -57,30 +58,95 @@ defmodule RabbitMQ.CLI.Core.Config do
   def normalise(:longnames, _val), do: :shortnames
   def normalise(_, value), do: value
 
-  def system_env_variable(name) do
-    case name do
-      :longnames -> "RABBITMQ_USE_LONGNAME"
-      :rabbitmq_home -> "RABBITMQ_HOME"
-      :mnesia_dir -> "RABBITMQ_MNESIA_DIR"
-      :plugins_dir -> "RABBITMQ_PLUGINS_DIR"
-      :plugins_expand_dir -> "RABBITMQ_PLUGINS_EXPAND_DIR"
-      :feature_flags_file -> "RABBITMQ_FEATURE_FLAGS_FILE"
-      :enabled_plugins_file -> "RABBITMQ_ENABLED_PLUGINS_FILE"
-      :node -> "RABBITMQ_NODENAME"
-      :aliases_file -> "RABBITMQ_CLI_ALIASES_FILE"
-      :erlang_cookie -> "RABBITMQ_ERLANG_COOKIE"
-      _ -> ""
-    end
-  end
-
-  def get_system_option(:script_name) do
+  def get_system_option(:script_name, _) do
     Path.basename(:escript.script_name())
     |> Path.rootname()
     |> String.to_atom()
   end
 
-  def get_system_option(name) do
-    System.get_env(system_env_variable(name))
+  def get_system_option(:aliases_file, _) do
+      System.get_env("RABBITMQ_CLI_ALIASES_FILE")
+  end
+
+  def get_system_option(:erlang_cookie, _) do
+      System.get_env("RABBITMQ_ERLANG_COOKIE")
+  end
+
+  def get_system_option(:node, %{offline: true} = opts) do
+      remote_node = case opts[:node] do
+        nil -> nil
+        val -> Helpers.normalise_node_option(val, opts[:longnames], opts)
+      end
+      context = get_env_context(remote_node, true)
+      get_val_from_env_context(context, :node)
+  end
+  def get_system_option(:node, opts) do
+      remote_node = case opts[:node] do
+        nil -> nil
+        val -> Helpers.normalise_node_option(val, opts[:longnames], opts)
+      end
+      context = get_env_context(remote_node, false)
+      get_val_from_env_context(context, :node)
+  end
+
+  def get_system_option(name, opts) do
+    work_offline = opts[:offline] == true
+    remote_node = case name do
+      :longnames -> nil
+      :rabbitmq_home -> nil
+      _ -> node_flag_or_default(opts)
+    end
+    context = get_env_context(remote_node, work_offline)
+    val0 = get_val_from_env_context(context, name)
+    val = cond do
+      remote_node != nil and
+      val0 == :undefined and
+      (name == :plugins_dir or name == :enabled_plugins_file) ->
+        context1 = get_env_context(nil, true)
+        get_val_from_env_context(context1, name)
+      true ->
+        val0
+    end
+    case val do
+      :undefined -> nil
+      _          -> val
+    end
+  end
+
+  def get_env_context(nil, _) do
+    :rabbit_env.get_context()
+  end
+  def get_env_context(remote_node, work_offline) do
+    case work_offline do
+      true  -> :rabbit_env.get_context(:offline)
+      false -> :rabbit_env.get_context(remote_node)
+    end
+  end
+
+  def get_val_from_env_context(context, name) do
+    case name do
+      :node -> context[:nodename]
+      :longnames -> context[:nodename_type] == :longnames
+      :rabbitmq_home -> context[:rabbitmq_home]
+      :mnesia_dir -> context[:mnesia_dir]
+      :plugins_dir -> context[:plugins_path]
+      :plugins_expand_dir -> context[:plugins_expand_dir]
+      :feature_flags_file -> context[:feature_flags_file]
+      :enabled_plugins_file -> context[:enabled_plugins_file]
+    end
+  end
+
+  def node_flag_or_default(opts) do
+    case opts[:node] do
+      nil ->
+        # Just in case `opts` was not normalized yet (to get the
+        # default node), we do it here as well.
+        case Helpers.normalise_node_option(opts) do
+          {:error, _}            -> nil
+          {:ok, normalized_opts} -> normalized_opts[:node]
+        end
+      node -> node
+    end
   end
 
   def default(:script_name), do: :rabbitmqctl
