@@ -54,7 +54,7 @@ defmodule RabbitMQ.CLI.Core.DocGuide do
   Macros.defguide("consumers")
   Macros.defguide("definitions")
   Macros.defguide("erlang_versions", path_segment: "which-erlang")
-  Macros.defguide("feature_flags", domain: "next.rabbitmq.com")
+  Macros.defguide("feature_flags")
   Macros.defguide("firehose")
   Macros.defguide("mirroring", path_segment: "ha")
   Macros.defguide("logging")
@@ -66,7 +66,7 @@ defmodule RabbitMQ.CLI.Core.DocGuide do
   Macros.defguide("publishers")
   Macros.defguide("plugins")
   Macros.defguide("queues")
-  Macros.defguide("quorum_queues", domain: "next.rabbitmq.com")
+  Macros.defguide("quorum_queues")
   Macros.defguide("runtime_tuning", path_segment: "runtime")
   Macros.defguide("tls", path_segment: "ssl")
   Macros.defguide("troubleshooting")
