@@ -14,10 +14,9 @@
 ## Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ClearTopicPermissionsCommand do
-  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
+  alias RabbitMQ.CLI.Core.{DocGuide, ExitCodes, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
 
   def merge_defaults(args, opts) do
     {args, Map.merge(%{vhost: "/"}, opts)}
@@ -53,6 +52,20 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ClearTopicPermissionsCommand do
     ])
   end
 
+  def output({:error, {:no_such_user, username}}, %{node: node_name, formatter: "json"}) do
+    {:error, %{"result" => "error", "node" => node_name, "message" => "User #{username} does not exist"}}
+  end
+  def output({:error, {:no_such_vhost, vhost}}, %{node: node_name, formatter: "json"}) do
+    {:error, %{"result" => "error", "node" => node_name, "message" => "Virtual host #{vhost} does not exist"}}
+  end
+  def output({:error, {:no_such_user, username}}, _) do
+    {:error, ExitCodes.exit_nouser(), "User #{username} does not exist"}
+  end
+  def output({:error, {:no_such_vhost, vhost}}, _) do
+    {:error, "Virtual host #{vhost} does not exist"}
+  end
+  use RabbitMQ.CLI.DefaultOutput
+
   def usage, do: "clear_topic_permissions [--vhost <vhost>] <username> [<exchange>]"
 
   def usage_additional() do
