@@ -76,12 +76,29 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
   def output(result, %{node: node_name, unit: unit}) when is_list(result) do
     m = result_map(result)
 
+    product_name_section = case m do
+      %{:product_name => product_name} when product_name != "" ->
+        ["Product name: #{product_name}"]
+      _ ->
+        []
+    end
+    product_version_section = case m do
+      %{:product_version => product_version} when product_version != "" ->
+        ["Product version: #{product_version}"]
+      _ ->
+        []
+    end
+
     runtime_section = [
       "#{bright("Runtime")}\n",
       "OS PID: #{m[:pid]}",
       "OS: #{m[:os]}",
       # TODO: format
       "Uptime (seconds): #{m[:uptime]}",
+    ] ++
+    product_name_section ++
+    product_version_section ++
+    [
       "RabbitMQ version: #{m[:rabbitmq_version]}",
       "Node name: #{node_name}",
       "Erlang configuration: #{m[:erlang_version]}",
@@ -98,7 +115,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
 
     data_directory_section = [
       "\n#{bright("Data directory")}\n",
-      "Node data directory: #{m[:data_directory]}"
+      "Node data directory: #{m[:data_directory]}",
+      "Raft data directory: #{m[:raft_data_directory]}"
     ]
 
     config_section = [
@@ -194,6 +212,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
     %{
       os: os_name(Keyword.get(result, :os)),
       pid: Keyword.get(result, :pid),
+      product_name: Keyword.get(result, :product_name) |> to_string,
+      product_version: Keyword.get(result, :product_version) |> to_string,
       rabbitmq_version: Keyword.get(result, :rabbitmq_version) |> to_string,
       erlang_version: Keyword.get(result, :erlang_version) |> to_string |> String.trim_trailing,
       uptime: Keyword.get(result, :uptime),
@@ -215,6 +235,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
       memory: Keyword.get(result, :memory) |> Enum.into(%{}),
 
       data_directory: Keyword.get(result, :data_directory) |> to_string,
+      raft_data_directory: Keyword.get(result, :raft_data_directory) |> to_string,
+
       config_files: Keyword.get(result, :config_files) |> Enum.map(&to_string/1),
       log_files: Keyword.get(result, :log_files) |> Enum.map(&to_string/1),
 
