@@ -20,7 +20,7 @@ defmodule RabbitMQCtl.MixfileBase do
     [
       app: :rabbitmqctl,
       version: "3.8.0-dev",
-      elixir: ">= 1.7.0 and < 1.11.0",
+      elixir: ">= 1.8.0 and < 1.11.0",
       build_embedded: Mix.env == :prod,
       start_permanent: Mix.env == :prod,
       escript: [main_module: RabbitMQCtl,
@@ -136,6 +136,18 @@ defmodule RabbitMQCtl.MixfileBase do
             compile: false,
             override: true
           },
+          {
+            :goldrush,
+            path: Path.join(deps_dir, "goldrush"),
+            compile: false,
+            override: true
+          },
+          {
+            :lager,
+            path: Path.join(deps_dir, "lager"),
+            compile: false,
+            override: true
+          },
           {
             :amqp_client,
             path: Path.join(deps_dir, "amqp_client"),
