@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ Federation.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_federation_link_util).
@@ -267,6 +267,9 @@ handle_upstream_down(Reason, _Args, State) ->
 
 %%----------------------------------------------------------------------------
 
+log_terminate(gone, _Upstream, _UParams, _XorQName) ->
+    %% the link cannot start, this has been logged already
+    ok;
 log_terminate({shutdown, restart}, _Upstream, _UParams, _XorQName) ->
     %% We've already logged this before munging the reason
     ok;
