@@ -52,7 +52,9 @@ to_json(ReqData, Context = #context{user = User = #user{tags = Tags}}) ->
                  {rates_mode,                RatesMode},
                  {sample_retention_policies, SRP},
                  {exchange_types,            ExchangeTypes},
-                 {rabbitmq_version,          version(rabbit)},
+                 {product_version,           list_to_binary(rabbit:product_version())},
+                 {product_name,              list_to_binary(rabbit:product_name())},
+                 {rabbitmq_version,          list_to_binary(rabbit:base_product_version())},
                  {cluster_name,              rabbit_nodes:cluster_name()},
                  {erlang_version,            erlang_version()},
                  {erlang_full_version,       erlang_full_version()},
