@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_web_stomp_handler).
@@ -234,6 +234,9 @@ handle_data(Data, State0) ->
     case handle_data1(Data, State0) of
         {ok, State1 = #state{state = blocked}} ->
             {[{active, false}], State1};
+        {error, Error0} ->
+            Error1 = rabbit_misc:format("~p", [Error0]),
+            stop(State0, 1007, Error1);
         Other ->
             Other
     end.
@@ -257,7 +260,9 @@ handle_data1(Bytes, State = #state{proc_state  = ProcState,
                                      connection  = ConnPid });
                 {stop, _Reason, ProcState1} ->
                     stop(State#state{ proc_state = ProcState1 })
-            end
+            end;
+        Other ->
+            Other
     end.
 
 maybe_block(State = #state{state = blocking, heartbeat = Heartbeat},
@@ -267,11 +272,14 @@ maybe_block(State = #state{state = blocking, heartbeat = Heartbeat},
 maybe_block(State, _) ->
     State.
 
-stop(State = #state{proc_state = ProcState}) ->
+stop(State) ->
+    stop(State, 1000, "STOMP died").
+
+stop(State = #state{proc_state = ProcState}, CloseCode, Error) ->
     maybe_emit_stats(State),
     ok = file_handle_cache:release(),
     rabbit_stomp_processor:flush_and_die(ProcState),
-    {reply, {close, 1000, "STOMP died"}, State}.
+    {[{close, CloseCode, Error}], State}.
 
 %%----------------------------------------------------------------------------
 
